@@ -6,6 +6,7 @@ use crate::display::display_optional;
 use crate::display::display_readonly;
 use crate::display::SliceDisplayer;
 use crate::interface::expr_to_name;
+use crate::params::pat_to_param_def;
 use crate::params::ts_fn_param_to_param_def;
 use crate::swc_util::is_false;
 use crate::ts_type_param::maybe_type_param_decl_to_type_param_defs;
@@ -673,7 +674,7 @@ impl From<&TsType> for TsTypeDef {
   }
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct TsTypeRefDef {
   pub type_params: Option<Vec<TsTypeDef>>,
@@ -690,7 +691,7 @@ pub enum LiteralDefKind {
   BigInt,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct LiteralDef {
   pub kind: LiteralDefKind,
@@ -708,14 +709,14 @@ pub struct LiteralDef {
   pub boolean: Option<bool>,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct TsTypeOperatorDef {
   pub operator: String,
   pub ts_type: TsTypeDef,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct TsFnOrConstructorDef {
   pub constructor: bool,
@@ -724,7 +725,58 @@ pub struct TsFnOrConstructorDef {
   pub type_params: Vec<TsTypeParamDef>,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+impl From<&deno_ast::swc::ast::ArrowExpr> for TsFnOrConstructorDef {
+  fn from(expr: &deno_ast::swc::ast::ArrowExpr) -> Self {
+    let params = expr
+      .params
+      .iter()
+      .map(|pat| pat_to_param_def(None, pat))
+      .collect();
+    let ts_type = expr
+      .return_type
+      .as_ref()
+      .map(ts_type_ann_to_def)
+      .unwrap_or_else(|| TsTypeDef::keyword("unknown"));
+    let type_params =
+      maybe_type_param_decl_to_type_param_defs(expr.type_params.as_ref());
+
+    Self {
+      constructor: false,
+      ts_type,
+      params,
+      type_params,
+    }
+  }
+}
+
+impl From<&deno_ast::swc::ast::FnExpr> for TsFnOrConstructorDef {
+  fn from(expr: &deno_ast::swc::ast::FnExpr) -> Self {
+    let params = expr
+      .function
+      .params
+      .iter()
+      .map(|param| pat_to_param_def(None, &param.pat))
+      .collect();
+    let ts_type = expr
+      .function
+      .return_type
+      .as_ref()
+      .map(ts_type_ann_to_def)
+      .unwrap_or_else(|| TsTypeDef::keyword("unknown"));
+    let type_params = maybe_type_param_decl_to_type_param_defs(
+      expr.function.type_params.as_ref(),
+    );
+
+    Self {
+      constructor: false,
+      ts_type,
+      params,
+      type_params,
+    }
+  }
+}
+
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct TsConditionalDef {
   pub check_type: Box<TsTypeDef>,
@@ -733,13 +785,13 @@ pub struct TsConditionalDef {
   pub false_type: Box<TsTypeDef>,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct TsInferDef {
   pub type_param: Box<TsTypeParamDef>,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct TsImportTypeDef {
   pub specifier: String,
@@ -749,7 +801,7 @@ pub struct TsImportTypeDef {
   pub type_params: Option<Vec<TsTypeDef>>,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct TsIndexedAccessDef {
   pub readonly: bool,
@@ -769,7 +821,7 @@ pub struct TsIndexedAccessDef {
 /// - `optional` = `None`
 /// - `ts_type` = `Some(TsTypeDef)` (`Type[Properties]`)
 ///
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct TsMappedTypeDef {
   #[serde(skip_serializing_if = "Option::is_none")]
@@ -783,7 +835,7 @@ pub struct TsMappedTypeDef {
   pub ts_type: Option<Box<TsTypeDef>>,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct LiteralMethodDef {
   pub name: String,
@@ -812,7 +864,7 @@ impl Display for LiteralMethodDef {
   }
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct LiteralPropertyDef {
   pub name: String,
@@ -834,7 +886,7 @@ impl Display for LiteralPropertyDef {
     Ok(())
   }
 }
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct LiteralCallSignatureDef {
   pub params: Vec<ParamDef>,
@@ -852,7 +904,7 @@ impl Display for LiteralCallSignatureDef {
   }
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct LiteralIndexSignatureDef {
   pub readonly: bool,
@@ -875,7 +927,7 @@ impl Display for LiteralIndexSignatureDef {
   }
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct TsTypeLiteralDef {
   pub methods: Vec<LiteralMethodDef>,
@@ -910,7 +962,7 @@ pub enum TsTypeDefKind {
   ImportType,
 }
 
-#[derive(Debug, Default, Serialize, Deserialize, Clone)]
+#[derive(Debug, Default, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct TsTypeDef {
   pub repr: String,
@@ -981,7 +1033,7 @@ pub struct TsTypeDef {
   pub import_type: Option<TsImportTypeDef>,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(tag = "type", rename_all = "camelCase")]
 pub enum ThisOrIdent {
   This,
@@ -1005,7 +1057,7 @@ impl From<&TsThisTypeOrIdent> for ThisOrIdent {
 ///                           ^^^^^^^ ^^^^^    ^^^^^^^^
 ///                           (1)     (2)      (3)
 /// ```
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct TsTypePredicateDef {
   /// (1) Whether the predicate includes `asserts` keyword or not
@@ -1241,11 +1293,23 @@ pub fn ts_type_ann_to_def(type_ann: &TsTypeAnn) -> TsTypeDef {
   }
 }
 
-pub fn infer_simple_ts_type_from_expr(
+pub fn infer_ts_type_from_expr(
   expr: &Expr,
   is_const: bool,
 ) -> Option<TsTypeDef> {
   match expr {
+    Expr::Array(arr_lit) => {
+      // e.g.) const n = ["a", 1];
+      infer_ts_type_from_arr_lit(arr_lit, false)
+    }
+    Expr::Arrow(expr) => {
+      // e.g.) const f = (a: string): void => {};
+      infer_ts_type_from_arrow_expr(expr)
+    }
+    Expr::Fn(expr) => {
+      // e.g.) const f = function a(a:string): void {};
+      infer_ts_type_from_fn_expr(expr)
+    }
     Expr::Lit(lit) => {
       // e.g.) const n = 100;
       infer_ts_type_from_lit(lit, is_const)
@@ -1258,6 +1322,10 @@ pub fn infer_simple_ts_type_from_expr(
       // e.g.) const s = `hello`;
       Some(infer_ts_type_from_tpl(tpl, is_const))
     }
+    Expr::TsConstAssertion(assertion) => {
+      // e.g.) const s = [] as const;
+      infer_ts_type_from_const_assertion(assertion)
+    }
     Expr::Call(expr) => {
       // e.g.) const value = Number(123);
       infer_ts_type_from_call_expr(expr)
@@ -1271,44 +1339,121 @@ pub fn infer_simple_ts_type_from_var_decl(
   is_const: bool,
 ) -> Option<TsTypeDef> {
   if let Some(init_expr) = &decl.init {
-    infer_simple_ts_type_from_expr(init_expr.as_ref(), is_const)
+    infer_ts_type_from_expr(init_expr.as_ref(), is_const)
   } else {
     None
   }
 }
 
+fn infer_ts_type_from_arr_lit(
+  arr_lit: &ArrayLit,
+  is_const: bool,
+) -> Option<TsTypeDef> {
+  let mut defs = Vec::new();
+  for expr in arr_lit.elems.iter().flatten() {
+    if expr.spread.is_none() {
+      if let Some(ts_type) = infer_ts_type_from_expr(&expr.expr, is_const) {
+        if !defs.contains(&ts_type) {
+          defs.push(ts_type);
+        }
+      } else {
+        // it is not a trivial type that can be inferred an so will infer an
+        // an any array.
+        return Some(TsTypeDef {
+          repr: "any[]".to_string(),
+          kind: Some(TsTypeDefKind::Array),
+          array: Some(Box::new(TsTypeDef::keyword("any"))),
+          ..Default::default()
+        });
+      }
+    } else {
+      // TODO(@kitsonk) we should recursively unwrap the spread here
+      return Some(TsTypeDef {
+        repr: "any[]".to_string(),
+        kind: Some(TsTypeDefKind::Array),
+        array: Some(Box::new(TsTypeDef::keyword("any"))),
+        ..Default::default()
+      });
+    }
+  }
+  match defs.len() {
+    1 => Some(TsTypeDef {
+      kind: Some(TsTypeDefKind::Array),
+      array: Some(Box::new(defs[0].clone())),
+      ..Default::default()
+    }),
+    2.. => {
+      let union = TsTypeDef {
+        kind: Some(TsTypeDefKind::Union),
+        union: Some(defs),
+        ..Default::default()
+      };
+      Some(TsTypeDef {
+        kind: Some(TsTypeDefKind::Array),
+        array: Some(Box::new(union)),
+        ..Default::default()
+      })
+    }
+    _ => None,
+  }
+}
+
+fn infer_ts_type_from_arrow_expr(expr: &ArrowExpr) -> Option<TsTypeDef> {
+  Some(TsTypeDef {
+    kind: Some(TsTypeDefKind::FnOrConstructor),
+    fn_or_constructor: Some(Box::new(expr.into())),
+    ..Default::default()
+  })
+}
+
+fn infer_ts_type_from_fn_expr(expr: &FnExpr) -> Option<TsTypeDef> {
+  Some(TsTypeDef {
+    kind: Some(TsTypeDefKind::FnOrConstructor),
+    fn_or_constructor: Some(Box::new(expr.into())),
+    ..Default::default()
+  })
+}
+
+fn infer_ts_type_from_const_assertion(
+  assertion: &TsConstAssertion,
+) -> Option<TsTypeDef> {
+  match &*assertion.expr {
+    Expr::Array(arr_lit) => {
+      // e.g.) const n = ["a", 1] as const;
+      infer_ts_type_from_arr_lit(arr_lit, true)
+    }
+    _ => infer_ts_type_from_expr(&*assertion.expr, true),
+  }
+}
+
 fn infer_ts_type_from_lit(lit: &Lit, is_const: bool) -> Option<TsTypeDef> {
   match lit {
     Lit::Num(num) => {
       if is_const {
         Some(TsTypeDef::number_literal(num))
       } else {
-        let repr = format!("{}", num.value);
-        Some(TsTypeDef::number_with_repr(&repr))
+        Some(TsTypeDef::number_with_repr("number"))
       }
     }
     Lit::Str(str_) => {
       if is_const {
         Some(TsTypeDef::string_literal(str_))
       } else {
-        let repr = str_.value.to_string();
-        Some(TsTypeDef::string_with_repr(&repr))
+        Some(TsTypeDef::string_with_repr("string"))
       }
     }
     Lit::Bool(bool_) => {
       if is_const {
         Some(TsTypeDef::bool_literal(bool_))
       } else {
-        let repr = bool_.value.to_string();
-        Some(TsTypeDef::bool_with_repr(&repr))
+        Some(TsTypeDef::bool_with_repr("boolean"))
       }
     }
     Lit::BigInt(bigint_) => {
       if is_const {
         Some(TsTypeDef::bigint_literal(bigint_))
       } else {
-        let repr = bigint_.value.to_string();
-        Some(TsTypeDef::bigint_with_repr(&repr))
+        Some(TsTypeDef::bigint_with_repr("bigint"))
       }
     }
     Lit::Regex(regex) => Some(TsTypeDef::regexp(regex.exp.to_string())),
@@ -1322,7 +1467,10 @@ fn infer_ts_type_from_new_expr(new_expr: &NewExpr) -> Option<TsTypeDef> {
       repr: ident.sym.to_string(),
       kind: Some(TsTypeDefKind::TypeRef),
       type_ref: Some(TsTypeRefDef {
-        type_params: None,
+        type_params: new_expr
+          .type_args
+          .as_ref()
+          .map(|init| maybe_type_param_instantiation_to_type_defs(Some(init))),
         type_name: ident.sym.to_string(),
       }),
       ..Default::default()
@@ -1333,7 +1481,7 @@ fn infer_ts_type_from_new_expr(new_expr: &NewExpr) -> Option<TsTypeDef> {
 
 fn infer_ts_type_from_call_expr(call_expr: &CallExpr) -> Option<TsTypeDef> {
   match &call_expr.callee {
-    ExprOrSuper::Expr(expr) => {
+    Callee::Expr(expr) => {
       if let Expr::Ident(ident) = expr.as_ref() {
         let sym = ident.sym.to_string();
         match sym.as_str() {
@@ -1374,7 +1522,17 @@ impl Display for TsTypeDef {
 
     let kind = self.kind.as_ref().unwrap();
     match kind {
-      TsTypeDefKind::Array => write!(f, "{}[]", &*self.array.as_ref().unwrap()),
+      TsTypeDefKind::Array => {
+        let array = self.array.as_ref().unwrap();
+        if matches!(
+          array.kind,
+          Some(TsTypeDefKind::Union) | Some(TsTypeDefKind::Intersection)
+        ) {
+          write!(f, "({})[]", &*self.array.as_ref().unwrap())
+        } else {
+          write!(f, "{}[]", &*self.array.as_ref().unwrap())
+        }
+      }
       TsTypeDefKind::Conditional => {
         let conditional = self.conditional_type.as_ref().unwrap();
         write!(
